@@ -5,25 +5,51 @@ for managing all changes within the state of a project. The backend touches almo
 the framework. To gain a better understanding of its larger role with the framework, refer to the documentation
 about the design of the framework (link).
 
-This file provides the definition of the functionality a backend should implement, but it does not provide 
-a concrete implementation. 
+This file provides the definition of the functionality a backend should implement, but it does not provide
+a concrete implementation.
 
-This file also contains the definition of how to configure and dynamically load a backend class. To denote a 
+This file also contains the definition of how to configure and dynamically load a backend class. To denote a
 concrete implementation, a python `module` and `class` must be provide. These should both be strings. They are
-used to load a class that inherits from the base `Backend` class. It also takes in a generic dictionary to be 
-passed as configuration into the dynamicall loaded class.
+used to load a class that inherits from the base `Backend` class. It also takes in a generic dictionary to be
+passed as configuration into the dynamically loaded class.
 
 """
-import inspect
+from dataclasses import dataclass, field
 from typing import Dict, Tuple, Any, List
 from pydantic import BaseModel
 
-from .components import Component_Difference, ComponentModel
+from core.constructs.components import Component_Difference, ComponentModel
+from core.constructs.resource import (
+    Resource_Reference_Difference,
+    ResourceModel,
+    Resource_Difference,
+)
+from core.constructs.resource_state import Resource_State
 
-from .resource import Resource_Reference_Difference, ResourceModel, Resource_Difference
-from .resource_state import Resource_State
+from core.utils.module_loader import import_class, ImportClassError, ImportModuleError
+from core.utils.exceptions import cdev_core_error
 
-from ..utils.module_loader import import_module
+
+###############################
+##### Exceptions
+###############################
+
+
+@dataclass
+class BackendError(cdev_core_error):
+    help_message: str = ""
+    help_resources: List[str] = field(default_factory=lambda: [])
+
+
+@dataclass
+class BackendInitializationError(BackendError):
+    help_message: str = ""
+    help_resources: List[str] = field(default_factory=lambda: [])
+
+
+###############################
+##### API
+###############################
 
 
 class Backend_Configuration(BaseModel):
@@ -76,7 +102,7 @@ class Backend:
         """
         raise NotImplementedError
 
-    def delete_resource_state(self, resource_state_uuid: str):
+    def delete_resource_state(self, resource_state_uuid: str) -> None:
         """
         Delete a resource state within this store state.
 
@@ -114,7 +140,6 @@ class Backend:
         """
         raise NotImplementedError
 
-
     def get_component(
         self, resource_state_uuid: str, component_name: str
     ) -> ComponentModel:
@@ -134,11 +159,10 @@ class Backend:
         """
         raise NotImplementedError
 
-
     def update_component(
         self, resource_state_uuid: str, component_difference: Component_Difference
-    ):
-        """Make an update to the component. 
+    ) -> None:
+        """Make an update to the component.
 
         Args:
             resource_state_uuid (str): The resource state that the change is in.
@@ -150,10 +174,9 @@ class Backend:
         """
         pass
 
-
     def get_component_uuid(self, resource_state_uuid: str, component_name: str) -> str:
         """
-        Get the unique namespace for this resource state and component. 
+        Get the unique namespace for this resource state and component.
 
         Args:
             resource_state_uuid (str): The resource state that this component will be in.
@@ -186,7 +209,7 @@ class Backend:
         Returns:
             transaction_token (str): The transaction token to be used by the mapper when deploying the resource. This token can be used to give to a cloud
             provider as a idempotency token.
-            namespace_token (str): A suffix that can be added to deployed cloud resources that acts as a namespace for the resource within the cloud. 
+            namespace_token (str): A suffix that can be added to deployed cloud resources that acts as a namespace for the resource within the cloud.
 
         Raises:
             ResourceStateDoesNotExist
@@ -201,8 +224,8 @@ class Backend:
         diff: Resource_Difference,
         transaction_token: str,
         cloud_output: Dict,
-        resolved_cloud_information: Dict={}
-    ):
+        resolved_cloud_information: Dict = {},
+    ) -> None:
         """
         Notify the resource state that all changes to a resource have completed successfully. This will cause the resource to
         update the state of the resource to the new state.
@@ -213,7 +236,7 @@ class Backend:
             diff (Resource_Difference): The desired change in the resource
             transaction_token (str): Identifying token representing what transaction is being completed
             cloud_output (Dict): Output information from the cloud provider
-            resolved_cloud_information (Dict): Information that was resolved from the cloud output of the component to deploy this change. 
+            resolved_cloud_information (Dict): Information that was resolved from the cloud output of the component to deploy this change.
 
 
         Raises:
@@ -230,7 +253,7 @@ class Backend:
         diff: Resource_Difference,
         transaction_token: str,
         failed_state: Dict,
-    ):
+    ) -> None:
         """
         Notify the resource state that an attempted change to a resource has failed. The provided failed state should encapsulate any needed information
         for a future mapper to recover the state of the resource back into a proper state.
@@ -255,7 +278,7 @@ class Backend:
         self,
         resource_state_uuid: str,
         diff: Resource_Reference_Difference,
-    ):
+    ) -> None:
         """
         Either reference or dereference a resource from a different component.
 
@@ -275,7 +298,7 @@ class Backend:
     # We can just delete the failure from our failed state (not recommended unless you know what you are doing because it can leave resources in the cloud)
     def change_failed_state_of_resource_change(
         self, resource_state_uuid: str, transaction_token: str, new_failed_state: Dict
-    ):
+    ) -> None:
         """
         Update the failed state of a 'resource change'.
 
@@ -296,7 +319,7 @@ class Backend:
         resource_state_uuid: str,
         transaction_token: str,
         to_previous_state: bool = True,
-    ):
+    ) -> None:
         """
         Recover the state of the change back to the previous state or forward to the new state. Note this will result in the failed resource change being removed from the stored state.
 
@@ -314,7 +337,7 @@ class Backend:
 
     def remove_failed_resource_change(
         self, resource_state_uuid: str, transaction_token: str
-    ):
+    ) -> None:
         """
         Completely remove the failed resource change from the stored state. Note that this should be used with caution as it can lead to hanging cloud resources.
 
@@ -375,6 +398,30 @@ class Backend:
         """
         raise NotImplementedError
 
+    # Api for getting a set of resources that match the given tag from the backend
+    def get_resources_by_tag(
+            self,
+            resource_state_uuid: str,
+            component_name: str,
+            resource_type: str,
+            resource_tag: str,
+    ) -> List[ResourceModel]:
+        """
+        Get the state of a resource from a component based on the name of the resource
+
+        Args:
+            resource_state_uuid (str): The resource state for this resource.
+            component_name (str): The component this resource is in.
+            resource_type: The RUUID of the resource desired
+            resource_tag: The tag of the resources desired
+
+        Raises:
+            ResourceStateDoesNotExist
+            ComponentDoesNotExist
+            ResourceDoesNotExist
+        """
+        raise NotImplementedError
+
     def get_cloud_output_value_by_name(
         self,
         resource_state_uuid: str,
@@ -403,7 +450,6 @@ class Backend:
         """
         raise NotImplementedError
 
-
     def get_cloud_output_by_name(
         self,
         resource_state_uuid: str,
@@ -469,7 +515,9 @@ class Backend:
         new_components: List[ComponentModel],
         old_components: List[str],
     ) -> Tuple[
-        Component_Difference,  Resource_Difference, Resource_Reference_Difference,
+        List[Component_Difference],
+        List[Resource_Difference],
+        List[Resource_Reference_Difference],
     ]:
         """
         Create the set of differences from a proposed set of components to a provided set of current components identified by their name. This allows the flexibility for working on a particular
@@ -493,33 +541,28 @@ def load_backend(config: Backend_Configuration) -> Backend:
         Backend: [description]
     """
     try:
-        backend_module = import_module(config.python_module)
-    except Exception as e:
-        print("Error loading backend module")
-        print(f"Error > {e}")
-
-        raise e
-
-    backend_class = None
-    for item in dir(backend_module):
-        potential_obj = getattr(backend_module, item)
-        if (
-            inspect.isclass(potential_obj)
-            and issubclass(potential_obj, Backend)
-            and item == config.python_class
-        ):
-            backend_class = potential_obj
-            break
-
-    if not backend_class:
-        print(f"Could not find {config.python_class} in {config.python_module}")
-        raise Exception
+        backend_class = import_class(config.python_module, config.python_class)
+    except ImportClassError as e:
+        raise BackendInitializationError(
+            error_message=f"""When loading '{config.python_class}' from '{config.python_module}' as the settings base class the following exception occurred:
+            {e.error_message}
+            """
+        )
+    except ImportModuleError as e:
+        raise BackendInitializationError(
+            error_message=f"""When loading '{config.python_module}' to load the base settings class ('{config.python_class}') the following exception occurred:
+            {e.error_message}
+            """
+        )
 
     try:
         # initialize the backend obj with the provided configuration values
-        initialized_obj = potential_obj(**config.config)
+        initialized_obj = backend_class(**config.config)
     except Exception as e:
-        print(f"Could not initialize {potential_obj} Class from config {config.config}")
-        raise e
+        raise BackendInitializationError(
+            error_message=f"""When Initializing Backend ('{config.python_module}.{config.python_class}') with Configuration {config.config}, an error occurred:
+            {e}
+            """
+        )
 
     return initialized_obj
